import { IRule, RuleProps } from '@aws-cdk/aws-events';
import { FunctionProps, IFunction } from '@aws-cdk/aws-lambda';
import { IQueue, QueueProps } from '@aws-cdk/aws-sqs';
import { Construct, Duration } from '@aws-cdk/core';
/**
 * @stability stable
 */
export interface IEventFunctionDeliveryProps {
    /**
     * The maximum age of a request that Lambda sends to a function for processing.
     *
     * @stability stable
     */
    maxEventAge?: Duration;
    /**
     * The maximum number of times to retry when the function returns an error.
     *
     * @stability stable
     */
    retryAttempts?: number;
}
/**
 * @stability stable
 */
export interface IEventFunctionProps {
    /**
     * The configuration and behaviour of the EventBridge Rule which links the EventBus and the Event Function Lambda.
     *
     * @stability stable
     * @readonly true
     */
    ruleProps: RuleProps;
    /**
     * Configuration of the Event Function Lambda.
     *
     * @stability stable
     * @readonly true
     */
    functionProps: FunctionProps;
    /**
     * Configuration of the Delivery properties to the Event Function Lambda.
     *
     * @stability stable
     * @readonly true
     */
    deliveryProps?: IEventFunctionDeliveryProps;
    /**
     * Configuration of the Delivery DLQ.
     *
     * @stability stable
     * @readonly true
     */
    deliveryDlqProps?: QueueProps;
    /**
     * Configuration of the Processing DLQ.
     *
     * @stability stable
     * @readonly true
     */
    processingDlqProps?: QueueProps;
}
/**
 * EventFunction.
 *
 * A Lambda Function, triggered by an EventBridge Event
 *
 * Includes additional resources:
 *    - Configurable EventBridge Rule
 *
 * @stability stable
 */
export declare class EventFunction extends Construct {
    /**
     * Lambda Function which is the target of the Event.
     *
     * @stability stable
     */
    readonly lambdaFunction: IFunction;
    /**
     * EventBridge Rule which controls when the Event Function is triggered.
     *
     * @stability stable
     */
    readonly rule: IRule;
    /**
     * The SQS Queue which acts as a DLQ for any EventBridge Delivery failures.
     *
     * @stability stable
     */
    readonly deliveryDlq: IQueue;
    /**
     * The SQS Queue which acts as a DLQ for any events which the Event Function is unable to process.
     *
     * @stability stable
     */
    readonly processingDlq: IQueue;
    /**
     * @stability stable
     */
    constructor(scope: Construct, id: string, props: IEventFunctionProps);
}
